
//
// Copyright (C) 2017 David Carpenter
// 
// Permission is hereby granted, free of charge, 
// to any person obtaining a copy of this software 
// and associated documentation files (the "Software"), 
// to deal in the Software without restriction, 
// including without limitation the rights to use, 
// copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit 
// persons to whom the Software is furnished to do so, 
// subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be 
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//


// everything_listview_test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>

void get_lvitem(HWND hwnd,int index)
{
	wchar_t buf[MAX_PATH];
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	LVITEMW lvItem = {0};
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(LVITEM) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct
	lvItem.mask = LVIF_TEXT | LVIF_STATE | LVIF_STATE | LVIF_IMAGE;
	lvItem.stateMask = 0xffffffff;
	lvItem.iItem = index;
	lvItem.iSubItem = 0;
	lvItem.cchTextMax = MAX_PATH;

	// Point to after LVITEM in the remote buffer
	lvItem.pszText = (LPTSTR)(lpRemoteBuffer + sizeof( LVITEM )); 

	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &lvItem, sizeof(LVITEM), NULL );

	// Send the message
	ret = SendMessage( hwnd, LVM_GETITEMW, 0, (LPARAM)lpRemoteBuffer);

	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &lvItem, sizeof(LVITEM), NULL );

	ReadProcessMemory( hProcess, (LPVOID)(lpRemoteBuffer + sizeof(LVITEM)), buf, (MAX_PATH * sizeof(wchar_t)), NULL );

	//Fix pszText to point to same offset in local buffer
	lvItem.pszText = buf;

	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	if (ret)	
	{
		printf("Item %d: %08x %S\n",index,lvItem.state,lvItem.pszText);
	}
	
}

void getitemrect(HWND hwnd,int index,COLORREF color)
{
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	RECT rect;
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(RECT) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct
	rect.left = LVIR_LABEL;

	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &rect, sizeof(RECT), NULL );

	// Send the message
	ret = SendMessage( hwnd, LVM_GETITEMRECT, index, (LPARAM)lpRemoteBuffer);

	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &rect, sizeof(RECT), NULL );
	
	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	if (ret)	
	{
		POINT pt;
		HDC hdc;
		int x;
		int y;
		int wide;
		int high;
		
		pt.x = rect.left;
		pt.y = rect.top;
		wide = rect.right - rect.left;
		high = rect.bottom - rect.top;
		
		ClientToScreen(hwnd,&pt);
		
		hdc = GetDC(0);
		
		for(x=0;x<wide;x++)
		{
			SetPixel(hdc,pt.x+x,pt.y,color);
		}
		for(x=0;x<wide;x++)
		{
			SetPixel(hdc,pt.x+x,pt.y+high-1,color);
		}
		
		for(y=0;y<high;y++)
		{
			SetPixel(hdc,pt.x,pt.y+y,color);
		}
		for(y=0;y<high;y++)
		{
			SetPixel(hdc,pt.x+wide-1,pt.y+y,color);
		}
		
		ReleaseDC(0,hdc);
	
		printf("Item %d: %d %d %d %d - %d x %d\n",index,rect.left,rect.top,rect.right,rect.bottom,rect.right - rect.left,rect.bottom - rect.top);
		
	}
	
}

void getviewrect(HWND hwnd)
{
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	RECT rect;
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(RECT) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct
	rect.left = LVIR_LABEL;

	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &rect, sizeof(RECT), NULL );

	// Send the message
	ret = SendMessage( hwnd, LVM_GETVIEWRECT, 0, (LPARAM)lpRemoteBuffer);

	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &rect, sizeof(RECT), NULL );
	
	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	if (ret)	
	{
		POINT pt;
		HDC hdc;
		int x;
		int y;
		int wide;
		int high;
		
		pt.x = rect.left;
		pt.y = rect.top;
		wide = rect.right - rect.left;
		high = rect.bottom - rect.top;
		
		ClientToScreen(hwnd,&pt);
		
		hdc = GetDC(0);
		
		for(x=0;x<wide;x++)
		{
			SetPixel(hdc,pt.x+x,pt.y,RGB(255,0,255));
		}
		for(x=0;x<wide;x++)
		{
			SetPixel(hdc,pt.x+x,pt.y+high-1,RGB(255,0,255));
		}
		
		for(y=0;y<high;y++)
		{
			SetPixel(hdc,pt.x,pt.y+y,RGB(255,0,255));
		}
		for(y=0;y<high;y++)
		{
			SetPixel(hdc,pt.x+wide-1,pt.y+y,RGB(255,0,255));
		}
		
		ReleaseDC(0,hdc);
	
		printf("view %d %d %d %d - %d x %d\n",rect.left,rect.top,rect.right,rect.bottom,rect.right - rect.left,rect.bottom - rect.top);
		
	}
	
}

void getorigin(HWND hwnd)
{
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	POINT pt;
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(POINT) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct

	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &pt, sizeof(POINT), NULL );

	// Send the message
	ret = SendMessage( hwnd, LVM_GETORIGIN, 0, (LPARAM)lpRemoteBuffer);

	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &pt, sizeof(POINT), NULL );
	
	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	if (ret)	
	{
		printf("origin %d %d\n",pt.x,pt.y);
		
	}
	
}

void getsubitemrect(HWND hwnd,int index,COLORREF color)
{
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	RECT rect;
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(RECT) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct
	rect.left = LVIR_LABEL;
	rect.top = index;

	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &rect, sizeof(RECT), NULL );

	// Send the message
	ret = SendMessage( hwnd, LVM_GETSUBITEMRECT, index, (LPARAM)lpRemoteBuffer);

	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &rect, sizeof(RECT), NULL );
	
	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	if (ret)	
	{
		POINT pt;
		HDC hdc;
		int x;
		int y;
		int wide;
		int high;
		
		pt.x = rect.left;
		pt.y = rect.top;
		wide = rect.right - rect.left;
		high = rect.bottom - rect.top;
		
		ClientToScreen(hwnd,&pt);
		
		hdc = GetDC(0);
		
		for(x=0;x<wide;x++)
		{
			SetPixel(hdc,pt.x+x,pt.y,color);
		}
		for(x=0;x<wide;x++)
		{
			SetPixel(hdc,pt.x+x,pt.y+high-1,color);
		}
		
		for(y=0;y<high;y++)
		{
			SetPixel(hdc,pt.x,pt.y+y,color);
		}
		for(y=0;y<high;y++)
		{
			SetPixel(hdc,pt.x+wide-1,pt.y+y,color);
		}
		
		ReleaseDC(0,hdc);
	
		printf("Item %d: %d %d %d %d - %d x %d\n",index,rect.left,rect.top,rect.right,rect.bottom,rect.right - rect.left,rect.bottom - rect.top);
		
	}
	
}

void hittest(HWND hwnd)
{
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	LVHITTESTINFO hittestinfo;
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(LVHITTESTINFO) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct
	GetCursorPos(&hittestinfo.pt);
	ScreenToClient(hwnd,&hittestinfo.pt);
	
//	hittestinfo.pt.x -= 100;
//	hittestinfo.pt.y -= 100;
	
	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &hittestinfo, sizeof(LVHITTESTINFO), NULL );

	// Send the message
	ret = SendMessage( hwnd, LVM_SUBITEMHITTEST, 0, (LPARAM)lpRemoteBuffer);

	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &hittestinfo, sizeof(LVHITTESTINFO), NULL );
	
	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	printf("hittest %d: %d, %d: flags %08x, index %d, subitem %d\n",ret,hittestinfo.pt.x,hittestinfo.pt.y,hittestinfo.flags,hittestinfo.iItem,hittestinfo.iSubItem);
}

void setitemstate(HWND hwnd)
{
	wchar_t buf[MAX_PATH];
	DWORD dwProcessID;
	HANDLE hProcess;
	BYTE *lpRemoteBuffer;
	LVITEMW lvItem = {0};
	LRESULT ret;

	// Get the process id owning the window
	GetWindowThreadProcessId( hwnd, &dwProcessID );

	// Open the process wih all access (You may not have the rights to do this)
	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID );

	// Allocate a buffer in the remote process
	lpRemoteBuffer = (BYTE*)VirtualAllocEx( hProcess, NULL, sizeof(LVITEM) + (MAX_PATH * sizeof(wchar_t)),MEM_COMMIT, PAGE_READWRITE );

	// Fill in the LVITEM struct, this is in your own process
	// Set the pszText member to somewhere in the remote buffer,
	// For the example I used the address imediately following the LVITEM stuct
	lvItem.stateMask = 0xffffffff;
	lvItem.state = LVIS_SELECTED;

	// Point to after LVITEM in the remote buffer
	lvItem.pszText = (LPTSTR)(lpRemoteBuffer + sizeof( LVITEM )); 

	// Copy the local LVITEM to the remote buffer
	WriteProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &lvItem, sizeof(LVITEM), NULL );

	// Send the message
//	ret = SendMessage( hwnd, LVM_SETITEMSTATE, index, (LPARAM)lpRemoteBuffer);

//	ret = SendMessage( hwnd, LVM_GETISEARCHSTRINGA, 0, (LPARAM)lvItem.pszText);
//	ret = SendMessage( hwnd, LVM_GETISEARCHSTRINGA, 0, (LPARAM)lvItem.pszText);
	ret = SendMessage( hwnd, LVM_GETITEMPOSITION, 1, (LPARAM)lvItem.pszText);


	// Read the struct back from the remote process into local buffer
	ReadProcessMemory( hProcess, (LPVOID)lpRemoteBuffer, &lvItem, sizeof(LVITEM), NULL );

	ReadProcessMemory( hProcess, (LPVOID)(lpRemoteBuffer + sizeof(LVITEM)), buf, (MAX_PATH * sizeof(wchar_t)), NULL );

	//Fix pszText to point to same offset in local buffer
	lvItem.pszText = buf;

	// Clean-up
	VirtualFreeEx( hProcess, (LPVOID)lpRemoteBuffer, 0, MEM_RELEASE ); 
	
	CloseHandle( hProcess );

	printf("%d %d\n",((POINT *)lvItem.pszText)->x,((POINT *)lvItem.pszText)->y);
	
}


int __cdecl _tmain(int argc, _TCHAR* argv[])
{
	HWND everything_listview_hwnd;
	POINT pt;
	
	for(;;)
	{
		GetCursorPos(&pt);
		
//		everything_listview_hwnd = FindWindow(L"EVERYTHING",0);

	everything_listview_hwnd = WindowFromPoint(pt);
	
	{
		char classname[256];
		GetClassNameA(everything_listview_hwnd,classname,256);
		printf("%s\n",classname);
		Sleep(250);
		continue;
	}
		
		if (everything_listview_hwnd)
		{
			everything_listview_hwnd = FindWindowEx(everything_listview_hwnd,NULL,L"SysListView32",0);
			
			if (everything_listview_hwnd)
			{
				// 
				/*
				{
					int i;
					int count;
					
					count = ListView_GetItemCount(everything_listview_hwnd);
					printf("Count %d\n",count);

					for(i=0;i<count;i++)
					{
						get_lvitem(everything_listview_hwnd,i);
					}

			//		setitemstate(everything_listview_hwnd,1);
				}
				*/
				
				
				{
					// test scroll
					ListView_Scroll(everything_listview_hwnd,0,52);
				}				
			
						/*
				{
					int view;
					
					view = ListView_GetView(everything_listview_hwnd);
					printf("view %d\n",view);
					view = ListView_SetView(everything_listview_hwnd,LV_VIEW_ICON);

			//		setitemstate(everything_listview_hwnd,1);
				}				
				*/
				
				/*
				{
					int i;
					int count;
					
					count = ListView_GetItemCount(everything_listview_hwnd);
					printf("Count %d\n",count);
					
//					ListView_SetView(everything_listview_hwnd,LV_VIEW_ICON);
					

//					for(i=0;i<count;i++)
//					{
//						getitemrect(everything_listview_hwnd,i);
//					}

					getitemrect(everything_listview_hwnd,0,RGB(255,0,255));
					getitemrect(everything_listview_hwnd,1,RGB(0,0,255));

				}
				*/

/*
				{
					getsubitemrect(everything_listview_hwnd,0,RGB(255,0,255));
					getsubitemrect(everything_listview_hwnd,1,RGB(0,0,255));

				}
*/				
				/*
				{
					int hotitem;
					
					hotitem = ListView_GetHotItem(everything_listview_hwnd);
					printf("hotitem %d\n",hotitem);
				}				
				*/
				/*
				{
					DWORD spacing;
					
					spacing = ListView_GetItemSpacing(everything_listview_hwnd,TRUE);
					printf("spacing %d %d\n",LOWORD(spacing),HIWORD(spacing));

					spacing = ListView_GetItemSpacing(everything_listview_hwnd,FALSE);
					printf("icon spacing %d %d\n",LOWORD(spacing),HIWORD(spacing));
				}				
				*/
			/*
				{
					ListView_SetView(everything_listview_hwnd,LV_VIEW_ICON);
					getviewrect(everything_listview_hwnd);
				}				
				*/
				/*
				{
					ListView_SetColumnWidth(everything_listview_hwnd,0,-1);
					Sleep(100);
				}						
				*/
				/*
				{
					ListView_SetView(everything_listview_hwnd,LV_VIEW_ICON);
					getorigin(everything_listview_hwnd);
				}*/				
/*
				{
					DWORD size;
					
					ListView_SetView(everything_listview_hwnd,LV_VIEW_ICON);

					size = ListView_ApproximateViewRect(everything_listview_hwnd,-1,-1,-1);
					printf("icon spacing %d %d\n",LOWORD(size),HIWORD(size));
					
					{
						POINT pt;
						HDC hdc;
						int x;
						int y;
						int wide;
						int high;
						
						pt.x = 0;
						pt.y = 0;
						wide = LOWORD(size);
						high = HIWORD(size);
						
						ClientToScreen(everything_listview_hwnd,&pt);
						
						hdc = GetDC(0);
						
						for(x=0;x<wide;x++)
						{
							SetPixel(hdc,pt.x+x,pt.y,RGB(255,0,255));
						}
						for(x=0;x<wide;x++)
						{
							SetPixel(hdc,pt.x+x,pt.y+high-1,RGB(255,0,255));
						}
						
						for(y=0;y<high;y++)
						{
							SetPixel(hdc,pt.x,pt.y+y,RGB(255,0,255));
						}
						for(y=0;y<high;y++)
						{
							SetPixel(hdc,pt.x+wide-1,pt.y+y,RGB(255,0,255));
						}		
					}			
				}		
				*/
				
//				ListView_SetExtendedListViewStyleEx(everything_listview_hwnd,LVS_EX_FULLROWSELECT|LVS_EX_HEADERDRAGDROP,LVS_EX_FULLROWSELECT|LVS_EX_HEADERDRAGDROP);		
//				ListView_SetExtendedListViewStyle(everything_listview_hwnd,LVS_EX_FULLROWSELECT|LVS_EX_HEADERDRAGDROP,LVS_EX_FULLROWSELECT|LVS_EX_HEADERDRAGDROP);		

//				SetWindowLong(everything_listview_hwnd,GWL_STYLE,GetWindowLong(everything_listview_hwnd,GWL_STYLE) & ~LVS_SINGLESEL);
		
//		printf("inv\n");
//				RedrawWindow(everything_listview_hwnd,0,0,RDW_INVALIDATE);
//				InvalidateRect(everything_listview_hwnd,0,FALSE);
			}
		}
		
		Sleep(1);
	}

	return 0;
}

